#include <cstdio>
#include <iostream>

#include "adlmidi.h"
#include "midi-gm.h"

using namespace std;

static void Analyze(int insno)
{
    unsigned char D[11];
    for(unsigned d=0; d<11; ++d)
       D[d] = OPL_GMdata(insno, d);
       
	printf("\
Instrument %d (%s):
        Mod  Car
Attack   %2d  %2d  (0..15)            Mod  Car
Decay    %2d  %2d  (0..15)  Waveform   %2d  %2d  (0=sine 1=saw/2 2=saw 3=saw/4)
Sustain  %2d%c %2d%c (0..15)  Freqmult   %2d  %2d  (0..15)
Release  %2d  %2d  (0..15)  ScaleLevel %2d  %2d  (0..3)
Volume   %2d  %2d  (0..63)
VolVib:   %c   %c      Modulation type: %s
PitchVib: %c   %c      Feedback: %d
ScaleEnv: %c   %c

",
	insno, MidiInsName[insno]

	,    D[4]>>4 ,     D[5]>>4  // atk
	,    D[8]    ,     D[9]     // wform
	,    D[4]&15 ,     D[5]&15  // dec
	,15-(D[6]>>4)         // susdepth
	,   (D[0]&32)?'Y':'-' // susflag
	,15-(D[7]>>4)         // susdepth
	,   (D[1]&32)?'Y':'-' // susflag
	,    D[0]&15 ,     D[1]&15  // freqmult
	,    D[6]&15 ,     D[7]&15  // rel
	,    D[2]>>6 ,     D[3]>>6  // scale level
	,63-(D[2]&63), 63-(D[3]&63) // vol

	,    (D[0]&128)?'Y':'-' // vib (am)
	,    (D[1]&128)?'Y':'-' // vib (am)
	
	,    (D[10]&1) ? "additive" : "modulation"
	
	,     (D[0]&64)?'Y':'-' // vib (pitch)
	,     (D[1]&64)?'Y':'-' // vib (pitch)
	
	,    (D[10]>>2)&7       // feedback strength
	
	,     (D[0]&16)?'Y':'-' // ksr
	,     (D[1]&16)?'Y':'-' // ksr

);

/*
        D00..D0B contains the adlib instrument specs packed like this:
        modulator:                                              carrier:
        D00=[freq.muliplier]+[?scale env.]*16+[?sustain]*32+    =D01
                [?pitch vib]*64+[?vol.vib]*128
        D02=[63-volume]+[levelscale&1]*128+[l.s.&2]*64          =D03
        D04=[attack]*16+[decay]                                 =D05
        D06=[15-sustain]*16+[release]                           =D07
        D08=[wave select]                                       =D09
        D0A=[modulation feedback]*2+[?additive synthesis]
        D0B=unused
*/
    
}

int main(int argc, const char *const *argv)
{
	for(unsigned gm=1; gm<=128; ++gm)
		Analyze(GM(gm));
	for(unsigned gp=35; gp<=81; ++gp)
		Analyze(GP(gp));
	return 0;
}
